 
#ifdef LICENSE

/*
 * (c) 1989-2018 Sasan Ardalan
 * 
 */


#endif
 
#ifdef SHORT_DESCRIPTION

Charge Pump Based on State Space Method

#endif
 
#ifdef PROGRAMMERS

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
       float   __halfVcc;
       float   __VTH;
       float   __iPu;
       float   __iPd;
       float   __vs1_init;
       float   __vs2_init;
       float   __vs1;
       float   __vs2;
       float   __dTime;
       float   __C1;
       float   __C2;
       double   __iPprev;
       double   __iP;
       int   __flag;
       int   __wflag;
       int   __total;
       int   __count;
       double   __vc;
       double   __t;
       double   __vctrlInit;
       double   __vctrl;
       double   __vcInit;
       double   __tp;
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define  halfVcc  (state_P->__halfVcc)
#define  VTH  (state_P->__VTH)
#define  iPu  (state_P->__iPu)
#define  iPd  (state_P->__iPd)
#define  vs1_init  (state_P->__vs1_init)
#define  vs2_init  (state_P->__vs2_init)
#define  vs1  (state_P->__vs1)
#define  vs2 (state_P->__vs2)
#define  dTime  (state_P->__dTime)
#define  C1  (state_P->__C1)
#define  C2  (state_P->__C2)
#define  iPprev  (state_P->__iPprev)
#define  iP  (state_P->__iP)
#define  flag	  (state_P->__flag)
#define  wflag	  (state_P->__wflag)
#define  total	  (state_P->__total)
#define  count	  (state_P->__count)
#define  vc  (state_P->__vc)
#define  t  (state_P->__t)
#define  vctrlInit  (state_P->__vctrlInit)
#define  vctrl  (state_P->__vctrl)
#define  vcInit  (state_P->__vcInit)
#define  tp  (state_P->__tp)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define upsig(DELAY) (*(( float   *)PIN(0,DELAY)))
#define dnsig(DELAY) (*(( float   *)PIN(1,DELAY)))

/*         
 *    OUTPUT BUFFER  DEFINES 
 */ 
#define y(delay) *( float   *)POUT(0,delay)

/*         
 *    PARAMETER DEFINES 
 */ 
#define R (param_P[0]->value.f)
#define C11 (param_P[1]->value.f)
#define C22 (param_P[2]->value.f)
#define UpCurrentValue (param_P[3]->value.f)
#define DnCurrentValue (param_P[4]->value.f)
#define fs (param_P[5]->value.f)
#define vdd (param_P[6]->value.f)
#define vss (param_P[7]->value.f)
/*-------------- BLOCK CODE ---------------*/
 int  
CHPStateSpace
(int run_state,block_Pt block_P)

{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

   	int i;
   	float sample;
    int numberOfSamples;
    float up,dn;
 
   

    float vout;


	double i10;		//  Branch (1) current at incremental time t0.
	double i11;		//  Branch (1) current at incremental time t1.
	double i20;		//  Branch (2) current at incremental time t0.
	double i21;		//  Branch (2) current at incremental time t1.
	
	
	int vkUp;		//  Signal indicating to increase control voltage 'vk'.
	int vkDn;		//  Signal indicating to decrease control voltage 'vk'.
	
		//  Working variables.
		//  dv1 and dv2 are computed by eval().
		//  vs1 and vs2 are changed by advance().

	double dv1;		//  Calculated incremental V1 voltage.
	double dv2;		//  Calculated incremental V2 voltage.
//STATE	double vs1;		//  Stored voltage (Q1/C1) on C1 at incremental time t0.
//STATE	double vs2;		//  Stored voltage (Q2/C2) on C2 at incremental time t0.



	
	
	
	//   State Space
	double tt;
	double tdelay=10e-9;
	double a;
	double v1;
	double invC2;
	double aInv;
	double coeff;
	double beta;


	
	double T_;

	double g1;
	double g2;

	double w2;
	double wz;
	double Ip;

	int n=100;


	


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = " R ";
     char   *ptype0 = "float";
     char   *pval0 = "10000";
     char   *pname0 = "R";
     char   *pdef1 = " C1  pF ";
     char   *ptype1 = "float";
     char   *pval1 = "100";
     char   *pname1 = "C11";
     char   *pdef2 = " C2  pF ";
     char   *ptype2 = "float";
     char   *pval2 = "10";
     char   *pname2 = "C22";
     char   *pdef3 = " UP Current micro amps ";
     char   *ptype3 = "float";
     char   *pval3 = "1";
     char   *pname3 = "UpCurrentValue";
     char   *pdef4 = " Down Current micro amps ";
     char   *ptype4 = "float";
     char   *pval4 = "1";
     char   *pname4 = "DnCurrentValue";
     char   *pdef5 = " Sampling Rate ";
     char   *ptype5 = "float";
     char   *pval5 = "1e12";
     char   *pname5 = "fs";
     char   *pdef6 = " VDD ";
     char   *ptype6 = "float";
     char   *pval6 = "1.0";
     char   *pname6 = "vdd";
     char   *pdef7 = " VSS ";
     char   *ptype7 = "float";
     char   *pval7 = "0.0";
     char   *pname7 = "vss";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);
KrnModelParam(indexModel88,1 ,pdef1,ptype1,pval1,pname1);
KrnModelParam(indexModel88,2 ,pdef2,ptype2,pval2,pname2);
KrnModelParam(indexModel88,3 ,pdef3,ptype3,pval3,pname3);
KrnModelParam(indexModel88,4 ,pdef4,ptype4,pval4,pname4);
KrnModelParam(indexModel88,5 ,pdef5,ptype5,pval5,pname5);
KrnModelParam(indexModel88,6 ,pdef6,ptype6,pval6,pname6);
KrnModelParam(indexModel88,7 ,pdef7,ptype7,pval7,pname7);

      }
break;
   


/*         
 *    OUTPUT BUFFER SYSTEM  INITS 
 */ 
 case OUTPUT_BUFFER_INIT: 
 {
 int indexOC = block_P->model_index;
     char   *ptypeOut0 = " float ";
     char   *pnameOut0 = " y ";
KrnModelConnectionOutput(indexOC,0 ,pnameOut0,ptypeOut0);
}
 break;

/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = " float ";
     char   *pnameIn0 = " upsig ";
     char   *ptypeIn1 = " float ";
     char   *pnameIn1 = " dnsig ";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
KrnModelConnectionInput(indexIC,1 ,pnameIn1,ptypeIn1);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
             halfVcc = 0 ;
        VTH = 0 ;
        iPu = 0 ;
        iPd = 0 ;
        vs1_init = 0 ;
        vs2_init = 0 ;
        vs1 = 0 ;
        vs2= 0 ;
        dTime = 0 ;
        C1 = 0 ;
        C2 = 0 ;
        iPprev = 0 ;
        iP = 0 ;
        flag	 = 0 ;
        wflag	 = 0 ;
        total	 = 0 ;
        count	 = 0 ;
        vc = 0 ;
        t = 0 ;
        vctrlInit = 0 ;
        vctrl = 0 ;
        vcInit = 0 ;
        tp = 0 ;



         
   if(NO_OUTPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 outputs expected; %d connected\n",
              STAR_NAME,NO_OUTPUT_BUFFERS());
	      return(201);
   }

   SET_CELL_SIZE_OUT(0,sizeof( float ));

         
   if(NO_INPUT_BUFFERS() != 2 ){
       fprintf(stdout,"%s:2 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof( float ));

   SET_CELL_SIZE_IN(1,sizeof( float ));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 

halfVcc = (vdd - vss) / 2.0;
VTH = (vdd + vss) / 2.0;

iPu=DnCurrentValue*1e-06;
iPd=UpCurrentValue*1e-06;

vs1 = vs1_init;		//  Stored voltage (Q1/C1) on C1 at incremental time t0.
vs2 = vs2_init;		//  Stored voltage (Q2/C2) on C2 at incremental time t0.

dTime = 1.0/fs;

C1=C11*1e-12;
C2=C22*1e-12;


vctrlInit=0;
vcInit=0;

vctrl=0;
vc=0;


flag=0;
iPprev=0;

t=0;
count=0;
wflag=1;
total=0;



break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 




for (numberOfSamples = MIN_AVAIL(); numberOfSamples > 0; --numberOfSamples) {

    IT_IN(0);
    IT_IN(1);



       up=upsig(0);
       dn=dnsig(0);

	   vkUp = (up > VTH);
	   vkDn = (dn  > VTH);
	   iP = (iPu * (double) vkUp) - (iPd * (double) vkDn);

   
	
		// can make more efficient later by calculating non-changin in init.

		a=(C1+C2)/(R*C1*C2);
		aInv = 1/a;
		w2=1.0/(R*C2);
		wz=1.0/(R*C1);
	//++++++++++++++++++++++++++++

	    if(iPprev == 0 && iP !=0.0 && flag==1) {
		    flag=0;
			t=0;
			tp=0;
		}


			Ip=iP;

			g1=0;
			g2=aInv;

			if(iP != 0.0 && iPprev ==0.0) {
	                 vctrlInit=vctrl;
					 vctrl=vc;
			}

			if(iP != 0.0) {

			   tp += dTime;

	           flag=0;
			   g1=exp(-a*t);
			   g2=aInv*(1-exp(-a*t));


			    vctrl = vctrlInit*(g1+wz*g2)+vcInit*w2*g2+(Ip/C2)*(g2+wz*(g1-1)/(a*a)+wz*t/a);
			    vc=vctrlInit*wz*g2+vcInit*(g1+w2*g2)+(Ip/C2)*(wz*(g1-1)/(a*a)+wz*t/a);
			}
			if (iP==0.0  && !flag) {

	                        vctrlInit=vctrl;
	                        vcInit=vc;
				flag=1;
			}
			if (iP==0.0 && flag) {
				tt=t-tp;
			    g1=exp(-a*tt);
			    g2=aInv*(1-exp(-a*tt));

				vctrl=vctrlInit*(g1+wz*g2)+vcInit*w2*g2;
				vc=vctrlInit*wz*g2+vcInit*(g1+w2*g2);

			}
			//printf("%le\t%le\n",t,vctrl);

	        iPprev=iP;


	       t += dTime;
		   total += dTime;	
	
		vs1 =vc;
		vs2 =vctrl;
		
        vout=vs2;
	 





   
   	   /*
	    * ready output buffer for sample
	    * check for overflow
	    */
	   if(IT_OUT(0)) {
				KrnOverflow("CHPStateSpace",0);
				return(99);
	   }
	   /*
	    * output the sample
	    */
	   y(0)=vout;


}
    


break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 



break;
}
return(0);
}
