 
#ifdef LICENSE

/*  DC Motor Model
    Copyright (C) 2012-2018 IONOTRONICS Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.ionotronics.com
    Las Vegas, NV
*/

#endif
 
#ifdef SHORT_DESCRIPTION

This block models a DC Motor (Z_Transform) using PWM

#endif
 
#ifdef PROGRAMMERS

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
       double   __ia;
       double   __w;
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define  ia  (state_P->__ia)
#define  w  (state_P->__w)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define Va(DELAY) (*(( float   *)PIN(0,DELAY)))
#define TL(DELAY) (*(( float   *)PIN(1,DELAY)))

/*         
 *    OUTPUT BUFFER  DEFINES 
 */ 
#define wout(delay) *( float   *)POUT(0,delay)
#define Ia(delay) *( float   *)POUT(1,delay)

/*         
 *    PARAMETER DEFINES 
 */ 
#define Ra (param_P[0]->value.f)
#define La (param_P[1]->value.f)
#define J (param_P[2]->value.f)
#define Bv (param_P[3]->value.f)
#define KaPhi (param_P[4]->value.f)
#define T (param_P[5]->value.f)
/*-------------- BLOCK CODE ---------------*/
 int  
dcmotor
(int run_state,block_Pt block_P)

{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

   	int i;
   	float sample;
   	int samples;
   	


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = " Armature Resistance (Ohms) ";
     char   *ptype0 = "float";
     char   *pval0 = "1";
     char   *pname0 = "Ra";
     char   *pdef1 = " Armature Inductance (Henry) ";
     char   *ptype1 = "float";
     char   *pval1 = "0.046";
     char   *pname1 = "La";
     char   *pdef2 = " Total Moment Inertia (kg-m**2) ";
     char   *ptype2 = "float";
     char   *pval2 = "0.093";
     char   *pname2 = "J";
     char   *pdef3 = " Viscous Friction Constant (Nt-m/rd/sec) ";
     char   *ptype3 = "float";
     char   *pval3 = "0.007";
     char   *pname3 = "Bv";
     char   *pdef4 = " Back Electromotive Force and Torque Constant (Volt/rd or Nt-m/Amper) ";
     char   *ptype4 = "float";
     char   *pval4 = "0.55";
     char   *pname4 = "KaPhi";
     char   *pdef5 = " Sampling Time Period (sec) ";
     char   *ptype5 = "float";
     char   *pval5 = "0.0001";
     char   *pname5 = "T";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);
KrnModelParam(indexModel88,1 ,pdef1,ptype1,pval1,pname1);
KrnModelParam(indexModel88,2 ,pdef2,ptype2,pval2,pname2);
KrnModelParam(indexModel88,3 ,pdef3,ptype3,pval3,pname3);
KrnModelParam(indexModel88,4 ,pdef4,ptype4,pval4,pname4);
KrnModelParam(indexModel88,5 ,pdef5,ptype5,pval5,pname5);

      }
break;
   


/*         
 *    OUTPUT BUFFER SYSTEM  INITS 
 */ 
 case OUTPUT_BUFFER_INIT: 
 {
 int indexOC = block_P->model_index;
     char   *ptypeOut0 = " float ";
     char   *pnameOut0 = " wout ";
     char   *ptypeOut1 = " float ";
     char   *pnameOut1 = " Ia ";
KrnModelConnectionOutput(indexOC,0 ,pnameOut0,ptypeOut0);
KrnModelConnectionOutput(indexOC,1 ,pnameOut1,ptypeOut1);
}
 break;

/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = " float ";
     char   *pnameIn0 = " Va ";
     char   *ptypeIn1 = " float ";
     char   *pnameIn1 = " TL ";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
KrnModelConnectionInput(indexIC,1 ,pnameIn1,ptypeIn1);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
             ia = 0 ;
        w = 0 ;



         
   if(NO_OUTPUT_BUFFERS() != 2 ){
       fprintf(stdout,"%s:2 outputs expected; %d connected\n",
              STAR_NAME,NO_OUTPUT_BUFFERS());
	      return(201);
   }

   SET_CELL_SIZE_OUT(0,sizeof( float ));

   SET_CELL_SIZE_OUT(1,sizeof( float ));

         
   if(NO_INPUT_BUFFERS() != 2 ){
       fprintf(stdout,"%s:2 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof( float ));

   SET_CELL_SIZE_IN(1,sizeof( float ));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 



break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 




/* read one sample from each input buffer and process them */
for(samples = MIN_AVAIL(); samples >0; --samples) {
       IT_IN(0);
       IT_IN(1);
       
       // VA(0) is the Armature Voltage 
       // TL(0) is the Load Torque
       
           ia=(La-Ra*T)/La*ia-KaPhi*T/La*w+Va(0)/La;
          w=KaPhi*T/J*ia+(J-Bv*T)/J*w -TL(0)/J;
       
     
   	   /*
	    * ready output buffer for sample
	    * check for overflow
	    */
	   if(IT_OUT(0)) {
				KrnOverflow("dcmotor",0);
				return(99);
	   }
	   /*
	    * output the sample angular velocity  
	    */
	   wout(0)=w*T;

	    
	   if(IT_OUT(1)) {
				KrnOverflow("dcmotor",0);
				return(99);
	   }
	   /*
	    * output the Armature Current
	    */
	   Ia(0)=ia*T;



}
    


break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 



break;
}
return(0);
}
