<BLOCK>
<LICENSE>
/*  DC Motor Model
    Copyright (C) 2012-2018 IONOTRONICS Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.ionotronics.com
    Las Vegas, NV
*/
</LICENSE>

 
<BLOCK_NAME>
dcmotor
</BLOCK_NAME> 


<DESC_SHORT>
This block models a DC Motor (Z_Transform) using PWM
</DESC_SHORT>

<COMMENTS>
<![CDATA[ 

/*
 * 		dcmotor()
 * Z-Domain Model DC Motor
 *
 * Generated by blockgen
 *
 * Floating Point Processor
 * Notes:
 *    This is a block generated from the floating point processor template
 *    It will input a sample, process it and output the processed
 *    sample.
 *    Note added another input and output buffer.
 * 
  
 *    
 * Programmer: Sasan Ardalan
 * Date: June 24, 2012
 * Modified: 
 * Reference:
 *
 *       SIMULATION OF PULSE WIDTH MODULATED (PWM) CHOPPER DRIVES USING
 *                             Z-TRANSFORMATION
 *                   Fatma GORBUZ EyIip AKPINAR
 *                        Dokuz Eyliil University
 *               Department of Electrical and Electronics Engineering
 *                  35100 Bomova, I m R , TURKEY
 *                           1998
 *
 * Va=Ra*Ia+La*dIa/dt + Eg
 * Eg=KaPhi*w
 * Te=TL+jdw/dt+Bv*w
 * Te=KaPhi*Ia
 *
 * Sample Parameters:
 * Ra=1 Ohm
 * La=46 mH
 * J=0.093 kgm**2
 * Bv=0.007 Nt-m/rd/sec
 * KaPhi=0.55 V/rad/sec.
 * Sampling Period: T=0.0001 sec
 * Reference Speed wref=80 rad/sec
 * 110V, 2.5hp,1800rpm separately excited
 *
 * Above From:
 *  Turk J Elec Engin, VOL.10, NO.3 2002
 * 
 *          Stability Analysis of a Closed-Loop Control for a Pulse
 *          Width Modulated DC Motor Drive
 *             Fatma GURBUZ, Eyup AKPINAR
 *             Dokuz Eylul University, Department of Electrical and Electronics Engineering,
 *            Tlnaztepe, Buca, 35160 _Izmir-TURKEY
 *                 e-mail: fatma.gurbuz@eee.deu.edu.tr
 *
 */
]]>
</COMMENTS> 

<DECLARATIONS> 

   	int i;
   	float sample;
   	int samples;
   	

</DECLARATIONS> 

<PARAMETERS>
        <PARAM>
               <DEF> Armature Resistance (Ohms) </DEF>
               <TYPE> float </TYPE>
               <NAME> Ra </NAME>
               <VALUE> 1 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Armature Inductance (Henry) </DEF>
               <TYPE> float </TYPE>
               <NAME> La </NAME>
               <VALUE> 0.046 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Total Moment Inertia (kg-m**2) </DEF>
               <TYPE> float </TYPE>
               <NAME> J </NAME>
               <VALUE> 0.093 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Viscous Friction Constant (Nt-m/rd/sec) </DEF>
               <TYPE> float </TYPE>
               <NAME> Bv </NAME>
               <VALUE> 0.007 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Back Electromotive Force and Torque Constant (Volt/rd or Nt-m/Amper) </DEF>
               <TYPE> float </TYPE>
               <NAME> KaPhi </NAME>
               <VALUE> 0.55 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Sampling Time Period (sec) </DEF>
               <TYPE> float </TYPE>
               <NAME> T </NAME>
               <VALUE> 0.0001 </VALUE>
        </PARAM>
        
        
        

</PARAMETERS>


<STATES>
	<STATE>
		<TYPE> double </TYPE>
		<NAME> ia </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> double </TYPE>
		<NAME> w </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	
	
	
	
</STATES>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> Va </NAME>
	</BUFFER>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> TL </NAME>
	</BUFFER>

</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> wout </NAME>
	</BUFFER>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> Ia </NAME>
	</BUFFER>


</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 


]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 




/* read one sample from each input buffer and process them */
for(samples = MIN_AVAIL(); samples >0; --samples) {
       IT_IN(0);
       IT_IN(1);
       
       // VA(0) is the Armature Voltage 
       // TL(0) is the Load Torque
       
           ia=(La-Ra*T)/La*ia-KaPhi*T/La*w+Va(0)/La;
          w=KaPhi*T/J*ia+(J-Bv*T)/J*w -TL(0)/J;
       
     
   	   /*
	    * ready output buffer for sample
	    * check for overflow
	    */
	   if(IT_OUT(0)) {
				KrnOverflow("dcmotor",0);
				return(99);
	   }
	   /*
	    * output the sample angular velocity  
	    */
	   wout(0)=w*T;

	    
	   if(IT_OUT(1)) {
				KrnOverflow("dcmotor",0);
				return(99);
	   }
	   /*
	    * output the Armature Current
	    */
	   Ia(0)=ia*T;



}
    

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 


]]>
</WRAPUP_CODE> 



</BLOCK> 

