 
#ifdef LICENSE

/*  DC Motor Model
    Copyright (C) 2012-2018 IONOTRONICS Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.ionotronics.com
    Las Vegas, NV
*/

#endif
 
#ifdef SHORT_DESCRIPTION

PID Controller

#endif
 
#ifdef PROGRAMMERS

Sasan Ardalan

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
       double   __lastErr;
       double   __errSum;
       FILE*   __dump_F;
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define  lastErr  (state_P->__lastErr)
#define  errSum  (state_P->__errSum)
#define  dump_F  (state_P->__dump_F)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define x(DELAY) (*(( float   *)PIN(0,DELAY)))
#define desired(DELAY) (*(( float   *)PIN(1,DELAY)))

/*         
 *    OUTPUT BUFFER  DEFINES 
 */ 
#define y(delay) *( float   *)POUT(0,delay)

/*         
 *    PARAMETER DEFINES 
 */ 
#define Kp (param_P[0]->value.f)
#define Ki (param_P[1]->value.f)
#define Kd (param_P[2]->value.f)
#define maxOut (param_P[3]->value.f)
#define minOut (param_P[4]->value.f)
#define pwmBypass (param_P[5]->value.d)
#define timeInterval (param_P[6]->value.f)
/*-------------- BLOCK CODE ---------------*/
 int  
pid
(int run_state,block_Pt block_P)

{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

   	int i;
   	double  theInput;
   	double  theOutput;
   	double  theError;
   	double dErr; 
   	double pwmControl;
   	int samples;
    double desiredVal;


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = " Kp ";
     char   *ptype0 = "float";
     char   *pval0 = "1";
     char   *pname0 = "Kp";
     char   *pdef1 = " Ki ";
     char   *ptype1 = "float";
     char   *pval1 = "1";
     char   *pname1 = "Ki";
     char   *pdef2 = " Kd ";
     char   *ptype2 = "float";
     char   *pval2 = "1";
     char   *pname2 = "Kd";
     char   *pdef3 = " Maximum for PWM";
     char   *ptype3 = "float";
     char   *pval3 = "100";
     char   *pname3 = "maxOut";
     char   *pdef4 = " Minimum for PWM";
     char   *ptype4 = "float";
     char   *pval4 = "-100";
     char   *pname4 = "minOut";
     char   *pdef5 = " PWM Bypass";
     char   *ptype5 = "int";
     char   *pval5 = "0";
     char   *pname5 = "pwmBypass";
     char   *pdef6 = " Time Interval ";
     char   *ptype6 = "float";
     char   *pval6 = "1";
     char   *pname6 = "timeInterval";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);
KrnModelParam(indexModel88,1 ,pdef1,ptype1,pval1,pname1);
KrnModelParam(indexModel88,2 ,pdef2,ptype2,pval2,pname2);
KrnModelParam(indexModel88,3 ,pdef3,ptype3,pval3,pname3);
KrnModelParam(indexModel88,4 ,pdef4,ptype4,pval4,pname4);
KrnModelParam(indexModel88,5 ,pdef5,ptype5,pval5,pname5);
KrnModelParam(indexModel88,6 ,pdef6,ptype6,pval6,pname6);

      }
break;
   


/*         
 *    OUTPUT BUFFER SYSTEM  INITS 
 */ 
 case OUTPUT_BUFFER_INIT: 
 {
 int indexOC = block_P->model_index;
     char   *ptypeOut0 = " float ";
     char   *pnameOut0 = " y ";
KrnModelConnectionOutput(indexOC,0 ,pnameOut0,ptypeOut0);
}
 break;

/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = " float ";
     char   *pnameIn0 = " x ";
     char   *ptypeIn1 = " float ";
     char   *pnameIn1 = " desired ";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
KrnModelConnectionInput(indexIC,1 ,pnameIn1,ptypeIn1);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
             lastErr = 0 ;
        errSum = 0 ;
        dump_F = 0 ;



         
   if(NO_OUTPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 outputs expected; %d connected\n",
              STAR_NAME,NO_OUTPUT_BUFFERS());
	      return(201);
   }

   SET_CELL_SIZE_OUT(0,sizeof( float ));

         
   if(NO_INPUT_BUFFERS() != 2 ){
       fprintf(stdout,"%s:2 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof( float ));

   SET_CELL_SIZE_IN(1,sizeof( float ));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 


dump_F=fopen("pid_dump.dat","w");


break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 


for(samples = MIN_AVAIL(); samples >0; --samples) {
       IT_IN(0);
       IT_IN(1);

       theInput=x(0);
       theError=desired(0)-theInput;
       desiredVal=desired(0);
       errSum += (theError * timeInterval);
       dErr = (theError - lastErr) / timeInterval;
       lastErr=theError;
       
       /*
        * Compute PID Output
        */
        theOutput = Kp * theError + Ki * errSum + Kd * dErr;
          
          
        if(theOutput > maxOut) 
                  pwmControl=1;
        else if(theOutput < minOut )
                  pwmControl=0;
        else
                  pwmControl = (theOutput -minOut)/(maxOut -minOut);
                  
         fprintf(dump_F,"%f %f %f %f %f  %f %f\n",theInput,  desiredVal, theError, dErr, errSum,theOutput,pwmControl);         

   	   /*
	    * ready output buffer for sample
	    * check for overflow
	    */
	   if(IT_OUT(0)) {
				KrnOverflow("pid",0);
				return(99);
	   }
	   /*
	    * output the sample
	    */
          

	   if(pwmBypass) 
	      y(0)=theOutput;
	   else
	      y(0)=pwmControl;


}
    


break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 
  fclose(dump_F);


break;
}
return(0);
}
