<BLOCK>
<LICENSE>
/*  DC Motor Model
    Copyright (C) 2012-2018 IONOTRONICS Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.ionotronics.com
    Las Vegas, NV
*/
</LICENSE>


<BLOCK_NAME>
pid
</BLOCK_NAME> 

<PROGRAMMERS>
Sasan Ardalan
</PROGRAMMERS>

<DESC_SHORT>
PID Controller
</DESC_SHORT>

<COMMENTS>
<![CDATA[ 

/*
 * 		pid()
 *
 * Generated by blockgen
 *
 * Floating Point Processor
 * Notes:
 *    This is a block generated from the floating point processor template
 *    It will input a sample, process it and output the processed
 *    sample.
 * 
 *    
 * Programmer: Sasan Ardalan
 * Date: July 4, 2012
 * Modified: 
 *
 * Reference:
 * Many good descriptions of PID in the literature also in Wikipedia.
 * 
 * For good C Code:
 *
 * http://brettbeauregard.com/blog/2011/04/improving-the-beginners-pid-introduction/
 *
 *
 */
]]>
</COMMENTS> 


<PARAMETERS>
        <PARAM>
               <DEF> Kp </DEF>
               <TYPE> float </TYPE>
               <NAME> Kp </NAME>
               <VALUE> 1 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Ki </DEF>
               <TYPE> float </TYPE>
               <NAME> Ki </NAME>
               <VALUE> 1 </VALUE>
        </PARAM>
        <PARAM>
               <DEF> Kd </DEF>
               <TYPE> float </TYPE>
               <NAME> Kd </NAME>
               <VALUE> 1 </VALUE>
        </PARAM>
       <PARAM>
                <DEF> Maximum for PWM</DEF>
                <TYPE> float </TYPE>
                <NAME> maxOut </NAME>
                <VALUE> 100 </VALUE>
       </PARAM>
       <PARAM>
                <DEF> Minimum for PWM</DEF>
                <TYPE> float </TYPE>
                <NAME> minOut </NAME>
                <VALUE> -100 </VALUE>
       </PARAM>
      
       <PARAM>
                <DEF> PWM Bypass</DEF>
                <TYPE> int </TYPE>
                <NAME> pwmBypass </NAME>
                <VALUE> 0 </VALUE>
       </PARAM>
       
       <PARAM>
                <DEF> Time Interval </DEF>
                <TYPE> float </TYPE>
                <NAME> timeInterval </NAME>
                <VALUE> 1 </VALUE>
       </PARAM>
       
       
       
       
       
</PARAMETERS>


<STATES>
	<STATE>
		<TYPE> double </TYPE>
		<NAME> lastErr </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> double </TYPE>
		<NAME> errSum </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> FILE* </TYPE>
		<NAME> dump_F </NAME>
		<VALUE> 0 </VALUE>
	</STATE>	
	
	
</STATES>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> x </NAME>
	</BUFFER>
	
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> desired </NAME>
	</BUFFER>
	
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> y </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<DECLARATIONS> 

   	int i;
   	double  theInput;
   	double  theOutput;
   	double  theError;
   	double dErr; 
   	double pwmControl;
   	int samples;
    double desiredVal;

</DECLARATIONS> 



<INIT_CODE>
<![CDATA[ 


dump_F=fopen("pid_dump.dat","w");

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


for(samples = MIN_AVAIL(); samples >0; --samples) {
       IT_IN(0);
       IT_IN(1);

       theInput=x(0);
       theError=desired(0)-theInput;
       desiredVal=desired(0);
       errSum += (theError * timeInterval);
       dErr = (theError - lastErr) / timeInterval;
       lastErr=theError;
       
       /*
        * Compute PID Output
        */
        theOutput = Kp * theError + Ki * errSum + Kd * dErr;
          
          
        if(theOutput > maxOut) 
                  pwmControl=1;
        else if(theOutput < minOut )
                  pwmControl=0;
        else
                  pwmControl = (theOutput -minOut)/(maxOut -minOut);
                  
         fprintf(dump_F,"%f %f %f %f %f  %f %f\n",theInput,  desiredVal, theError, dErr, errSum,theOutput,pwmControl);         

   	   /*
	    * ready output buffer for sample
	    * check for overflow
	    */
	   if(IT_OUT(0)) {
				KrnOverflow("pid",0);
				return(99);
	   }
	   /*
	    * output the sample
	    */
          

	   if(pwmBypass) 
	      y(0)=theOutput;
	   else
	      y(0)=pwmControl;


}
    

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 
  fclose(dump_F);

]]>
</WRAPUP_CODE> 



</BLOCK> 

