 
#ifdef LICENSE

/*  PWM Generator
    Copyright (C) 2012-2018 IONOTRONICS Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.ionotronics.com
    Las Vegas, NV
*/


#endif
 
#ifdef SHORT_DESCRIPTION

This block generates a pwm pulse train with pulse width duty cycle controlled by input. 

#endif
 
#ifdef PROGRAMMERS

#endif






#include <stdio.h>
#include <math.h>

/* Note: these paths are installation dependent! */
#include <capsim.h>
#include <stars.h>



 

#include <math.h>


 

#define PI 3.14159265358979323846


/*
 *           STATES STRUCTURE 
 */ 
typedef struct {
      int  __samplesPerPeriod;
      int  __sampleCount;
      int  __numberInputBuffers;
      int  __no_outbuf;
     } state_t,*state_Pt;

/*
 *         STATE DEFINES 
 */ 
#define samplesPerPeriod (state_P->__samplesPerPeriod)
#define sampleCount (state_P->__sampleCount)
#define numberInputBuffers (state_P->__numberInputBuffers)
#define no_outbuf (state_P->__no_outbuf)

/*         
 *    INPUT BUFFER DEFINES 
 */ 
#define pulseWidth(DELAY) (*(( float   *)PIN(0,DELAY)))

/*         
 *    PARAMETER DEFINES 
 */ 
#define magnitude (param_P[0]->value.f)
#define freq (param_P[1]->value.f)
#define fs (param_P[2]->value.f)
#define dcOffset (param_P[3]->value.f)
/*-------------- BLOCK CODE ---------------*/
 int  

pwm 

(int run_state,block_Pt block_P)

{
	param_Pt *param_P = block_P->param_AP;
	star_Pt star_P = block_P->star_P;

     
	state_Pt state_P = (state_Pt)star_P->state_P;
     
/*
 *              Declarations 
 */
 

     int i,j;
     int pulseWidthSamples;


switch (run_state) {

 
 


 
case PARAM_INIT: 
    {
      int indexModel88 = block_P->model_index;
     char   *pdef0 = "Magnitude";
     char   *ptype0 = "float";
     char   *pval0 = "1.0";
     char   *pname0 = "magnitude";
     char   *pdef1 = "Frequency (Hz)";
     char   *ptype1 = "float";
     char   *pval1 = "1000";
     char   *pname1 = "freq";
     char   *pdef2 = "Sampling Rate (Hz)";
     char   *ptype2 = "float";
     char   *pval2 = "32000";
     char   *pname2 = "fs";
     char   *pdef3 = "DC offset";
     char   *ptype3 = "float";
     char   *pval3 = "0";
     char   *pname3 = "dcOffset";
KrnModelParam(indexModel88,0 ,pdef0,ptype0,pval0,pname0);
KrnModelParam(indexModel88,1 ,pdef1,ptype1,pval1,pname1);
KrnModelParam(indexModel88,2 ,pdef2,ptype2,pval2,pname2);
KrnModelParam(indexModel88,3 ,pdef3,ptype3,pval3,pname3);

      }
break;
   


/*         
 *    INPUT BUFFER SYSTEM  INITS 
 */ 
 case INPUT_BUFFER_INIT:
 {
 int indexIC = block_P->model_index;
     char   *ptypeIn0 = " float ";
     char   *pnameIn0 = " pulseWidth ";
KrnModelConnectionInput(indexIC,0 ,pnameIn0,ptypeIn0);
}
 break;

/*
 *        SYSTEM INITIALIZATION CODE 
 */
case SYSTEM_INIT:
     
	star_P->state_P = (char*)calloc(1,sizeof(state_t));
	state_P = (state_Pt)star_P->state_P;
     

         
   if(NO_INPUT_BUFFERS() != 1 ){
       fprintf(stdout,"%s:1 inputs expected; %d connected\n",
              STAR_NAME,NO_INPUT_BUFFERS());
	      return(200);
   }

   SET_CELL_SIZE_IN(0,sizeof( float ));

break;

/*
 *               USER INITIALIZATION CODE 
 */
case USER_INIT: 

 


        /*
         * Support Auto Fan-Out
         */
        if((no_outbuf = NO_OUTPUT_BUFFERS() ) <= 0) {
                fprintf(stderr,"pwm: no output buffers\n");
                return(1); /* no output buffers */
        }
        
        samplesPerPeriod=(int)(fs/freq);
        
        printf("pwm: Samples per period: %d\n",samplesPerPeriod);
        sampleCount=0;
        


break;

/* 
 *             MAIN CODE 
 */
case MAIN_CODE: 

 


       
    while(IT_IN(0)) {
            // current sample: pulseWidth(0)
        
            pulseWidthSamples=(int)(pulseWidth(0)*(1.0/freq)*fs+0.5);
            
            
	    for(j=0; j<no_outbuf; j++) {
		    if(IT_OUT(j)) {
			      KrnOverflow("pulse",j);	
			    return(99);
		    }
		    if ( sampleCount < pulseWidthSamples) 
				      OUTF(j,0) = magnitude + dcOffset;
		     else
				       OUTF(j,0) = 0.0+dcOffset;
        }        
        
            sampleCount =(sampleCount+1)%samplesPerPeriod;
        
	}

    return(0);



break;

/*             
 *            WRAPUP CODE 
 */
case WRAPUP: 

 



break;
}
return(0);
}
