<BLOCK>
<LICENSE>
/*  PWM Generator
    Copyright (C) 2012-2018 IONOTRONICS Corporation 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    http://www.ionotronics.com
    Las Vegas, NV
*/

</LICENSE>


<BLOCK_NAME>

pwm 

</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

/* pwm.s */
/***************************************************************************
                          pwm()
*****************************************************************************
This block generates a pwm pulse train. 


The first parameter, which defaults to NUMBER_SAMPLES_PER_VISIT, tells how many total samples
to send out.
The second  parameter is the magnitude which defauls to one.
The following parameters specify the pulse delay, width and period:
 np=period in samples
 nw=pulse width in samples
 nd=delay in samples

Programmer:	Sasan Ardalan
Date:		Initial: Nov. 1987, Adapted to PWM by Sasan Ardalan July 4, 2012
<NAME>
pulse
</NAME>

<DESCRIPTION>
This block generates a PWM signal. 

</DESCRIPTION>

<PROGRAMMERS>
Sasan Ardalan
</PROGRAMMERS>
*****************************************************************************/

]]>
</COMMENTS> 

<DESC_SHORT>
This block generates a pwm pulse train with pulse width duty cycle controlled by input. 
</DESC_SHORT>


<INCLUDES>
<![CDATA[ 

#include <math.h>

]]>
</INCLUDES> 

<DEFINES> 

#define PI 3.14159265358979323846

</DEFINES> 

         

<STATES>
	<STATE>
		<TYPE>int</TYPE>
		<NAME>samplesPerPeriod</NAME>
	</STATE>
	
	<STATE>
		<TYPE>int</TYPE>
		<NAME>sampleCount</NAME>
	</STATE>
	
	<STATE>
		<TYPE>int</TYPE>
		<NAME>numberInputBuffers</NAME>
	</STATE>
	<STATE>
		<TYPE>int</TYPE>
		<NAME>no_outbuf</NAME>
	</STATE>
</STATES>
 
<DECLARATIONS> 

     int i,j;
     int pulseWidthSamples;

</DECLARATIONS> 

                   

<PARAMETERS>
<PARAM>
	<DEF>Magnitude</DEF>
	<TYPE>float</TYPE>
	<NAME>magnitude</NAME>
	<VALUE>1.0</VALUE>
</PARAM>
<PARAM>
	<DEF>Frequency (Hz)</DEF>
	<TYPE>float</TYPE>
	<NAME>freq</NAME>
	<VALUE>1000</VALUE>
</PARAM>
<PARAM>
	<DEF>Sampling Rate (Hz)</DEF>
	<TYPE>float</TYPE>
	<NAME>fs</NAME>
	<VALUE>32000</VALUE>
</PARAM>

<PARAM>
	<DEF>DC offset</DEF>
	<TYPE>float</TYPE>
	<NAME>dcOffset</NAME>
	<VALUE>0</VALUE>
</PARAM>

</PARAMETERS>


<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> pulseWidth </NAME>
	</BUFFER>

</INPUT_BUFFERS>




<INIT_CODE>
<![CDATA[ 


        /*
         * Support Auto Fan-Out
         */
        if((no_outbuf = NO_OUTPUT_BUFFERS() ) <= 0) {
                fprintf(stderr,"pwm: no output buffers\n");
                return(1); /* no output buffers */
        }
        
        samplesPerPeriod=(int)(fs/freq);
        
        printf("pwm: Samples per period: %d\n",samplesPerPeriod);
        sampleCount=0;
        

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


       
    while(IT_IN(0)) {
            // current sample: pulseWidth(0)
        
            pulseWidthSamples=(int)(pulseWidth(0)*(1.0/freq)*fs+0.5);
            
            
	    for(j=0; j<no_outbuf; j++) {
		    if(IT_OUT(j)) {
			      KrnOverflow("pulse",j);	
			    return(99);
		    }
		    if ( sampleCount < pulseWidthSamples) 
				      OUTF(j,0) = magnitude + dcOffset;
		     else
				       OUTF(j,0) = 0.0+dcOffset;
        }        
        
            sampleCount =(sampleCount+1)%samplesPerPeriod;
        
	}

    return(0);


]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 


]]>
</WRAPUP_CODE> 



</BLOCK> 

